@@ -1,5 +1,3 @@
-/* To do: extract the iterative solving of the loop jumps' size.  */
-
 /******************************** -*- C -*- ****************************
  *
  *	Byte code compiler.
@@ -9,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -27,12 +25,14 @@
  * 
  * You should have received a copy of the GNU General Public License along with
  * GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
+ * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  *
  ***********************************************************************/
 
 #include "gstpriv.h"
 
+/* To do: extract the iterative solving of the loop jumps' size.  */
+
 /* Define this if you want declaration tracing to print the bytecodes
    both *before* and *after* the optimizer is ran.  Default behavior
    is to print the bytecodes only after the optimization pass; usually
@@ -288,15 +288,6 @@ static bc_vector compile_sub_expression (tree_node expr);
 static bc_vector compile_sub_expression_and_jump (tree_node expr,
 						  int branchLen);
 
-/* Scans the constants that are referenced by the current method.  If
-   one is found that is equal to CONSTEXPR, the index of that constant
-   is returned.  Otherwise, the constant is turned into a constant
-   object, added to the constants of the method, and the new index is
-   returned.
-
-   For positive integers, return -1.  */
-static int add_constant (tree_node constExpr);
-
 /* Compile a send with the given RECEIVER (used to check for sends to
    super), SELECTOR and number of arguments NUMARGS.  */
 static void compile_send (tree_node receiver,
@@ -485,8 +476,8 @@ _gst_install_initial_methods (void)
   /* Define the termination method first of all, because
      compiling #methodsFor: will invoke an evaluation
      (to get the argument of the <primitive: ...> attribute.  */
-  _gst_set_compilation_category (_gst_undefined_object_class,
-				 _gst_string_new ("private"));
+  _gst_set_compilation_class (_gst_undefined_object_class);
+  _gst_set_compilation_category (_gst_string_new ("private"));
   _gst_alloc_bytecodes ();
   _gst_compile_byte (EXIT_INTERPRETER, 0);
   _gst_compile_byte (RETURN_CONTEXT_STACK_TOP, 0);
@@ -506,13 +497,10 @@ methodsFor: aCategoryString \
       The methods are put in the category identified by the parameter.\" \
     <primitive: VMpr_Behavior_methodsFor> \
 ";
-  _gst_compile_code = true;	/* tell the lexer we do internal
-				   compiles */
-
-  _gst_set_compilation_category (_gst_behavior_class,
-				 _gst_string_new ("compiling methods"));
+  _gst_set_compilation_class (_gst_behavior_class);
+  _gst_set_compilation_category (_gst_string_new ("compiling methods"));
   _gst_push_smalltalk_string (_gst_string_new (methodsForString));
-  _gst_parse_stream ();
+  _gst_parse_stream (true);
   _gst_pop_stream (true);
 
   _gst_reset_compilation_category ();
@@ -558,15 +546,21 @@ _gst_init_compiler (void)
 }
 
 void
-_gst_set_compilation_category (OOP class_oop, OOP categoryOOP)
+_gst_set_compilation_class (OOP class_oop)
+{
+  _gst_unregister_oop (_gst_this_class);
+  _gst_this_class = class_oop;
+  _gst_register_oop (_gst_this_class);
+
+  _gst_untrusted_methods = (IS_OOP_UNTRUSTED (_gst_this_context_oop)
+			    || IS_OOP_UNTRUSTED (_gst_this_class));
+}
+
+void
+_gst_set_compilation_category (OOP categoryOOP)
 {
   _gst_unregister_oop (_gst_this_category);
-  _gst_unregister_oop (_gst_this_class);
-
-  _gst_this_class = class_oop;
   _gst_this_category = categoryOOP;
-
-  _gst_register_oop (_gst_this_class);
   _gst_register_oop (_gst_this_category);
 
   _gst_untrusted_methods = (IS_OOP_UNTRUSTED (_gst_this_context_oop)
@@ -576,7 +570,8 @@ _gst_set_compilation_category (OOP class_oop, OOP categoryOOP)
 void
 _gst_reset_compilation_category ()
 {
-  _gst_set_compilation_category (_gst_undefined_object_class, _gst_nil_oop);
+  _gst_set_compilation_class (_gst_undefined_object_class);
+  _gst_set_compilation_category (_gst_nil_oop);
   _gst_untrusted_methods = false;
 }
 
@@ -622,7 +617,8 @@ _gst_execute_statements (tree_node temporaries,
   _gst_register_oop (oldClass);
   _gst_register_oop (oldCategory);
 
-  _gst_set_compilation_category (_gst_undefined_object_class, _gst_nil_oop);
+  _gst_set_compilation_class (_gst_undefined_object_class);
+  _gst_set_compilation_category (_gst_nil_oop);
 
   messagePattern = _gst_make_unary_expr (&statements->location,
 					 NULL,
@@ -643,7 +639,8 @@ _gst_execute_statements (tree_node temporaries,
   SET_CLASS_ENVIRONMENT (_gst_undefined_object_class,
 			 _gst_smalltalk_dictionary);
 
-  _gst_set_compilation_category (oldClass, oldCategory);
+  _gst_set_compilation_class (oldClass);
+  _gst_set_compilation_category (oldCategory);
   _gst_unregister_oop (oldClass);
   _gst_unregister_oop (oldCategory);
 
@@ -1057,17 +1054,43 @@ compile_variable (tree_node varName)
 void
 compile_constant (tree_node constExpr)
 {
-  int index = add_constant (constExpr);
-  int value;
+  intptr_t intVal;
+  int index = -1;
+  OOP constantOOP;
+  OOP *lit;
+
+  /* Scan the current literal frame, looking for a constant equal
+     to the one that is being compiled.  */
+  for (lit = literal_vec; lit < literal_vec_curr; lit++)
+    if (equal_constant (*lit, constExpr))
+      {
+	index = lit - literal_vec;
+	break;
+      }
+
+  /* If not found, check if it can be compiled with a PUSH_INTEGER
+     bytecode, or add it to the literals.  */
+  if (index == -1)
+    {
+      constantOOP = make_constant_oop (constExpr);
+      if (IS_INT (constantOOP))
+        {
+          intVal = TO_INT (constantOOP);
+          if (intVal >= 0 && intVal <= 0x7FFFFFFFL)
+            {
+	      INCR_STACK_DEPTH ();
+	      _gst_compile_byte (PUSH_INTEGER, intVal);
+	      return;
+	    }
+        }
+      else
+        MAKE_OOP_READONLY (constantOOP, true);
+
+      index = add_literal (constantOOP);
+    }
 
   INCR_STACK_DEPTH ();
-  if COMMON (index != -1)
-    _gst_compile_byte (PUSH_LIT_CONSTANT, index);
-  else
-    {
-      value = constExpr->v_const.val.iVal;
-      _gst_compile_byte (PUSH_INTEGER, value);
-    }
+  _gst_compile_byte (PUSH_LIT_CONSTANT, index);
 }
 
 void
@@ -1437,6 +1460,10 @@ compile_send (tree_node expr,
       EXIT_COMPILATION ();
     }
 
+  if (super)
+    compile_constant (_gst_make_oop_constant (&expr->location,
+					      SUPERCLASS (_gst_this_class)));
+
   if (!bs)
     {
       int selectorIndex = _gst_add_forced_object (selector);
@@ -1975,30 +2002,6 @@ is_super (tree_node expr)
 }
 
 
-int
-add_constant (tree_node constExpr)
-{
-  intptr_t intVal;
-  OOP constantOOP;
-  OOP *lit;
-
-  for (lit = literal_vec; lit < literal_vec_curr; lit++)
-    if (equal_constant (*lit, constExpr))
-      return (lit - literal_vec);
-
-  constantOOP = make_constant_oop (constExpr);
-  if (IS_INT (constantOOP))
-    {
-      intVal = TO_INT (constantOOP);
-      if (intVal >= 0 && intVal < INT_MAX)
-	return (-1);
-    }
-  else
-    MAKE_OOP_READONLY (constantOOP, true);
-
-  return (add_literal (constantOOP));
-}
-
 mst_Boolean
 equal_constant (OOP oop,
 		tree_node constExpr)
@@ -2042,22 +2045,37 @@ equal_constant (OOP oop,
 	return (true);
       break;
 
-    case CONST_FLOATD:
-      if (IS_OOP (oop) && OOP_CLASS (oop) == _gst_floatd_class
-	  && (double) constExpr->v_const.val.fVal == FLOATD_OOP_VALUE (oop))
+    case CONST_CHAR:
+      if (IS_OOP (oop) && is_a_kind_of (OOP_CLASS (oop), _gst_char_class)
+	  && CHAR_OOP_VALUE (oop) == constExpr->v_const.val.iVal)
 	return (true);
       break;
 
+    case CONST_FLOATD:
+      if (IS_OOP (oop) && OOP_CLASS (oop) == _gst_floatd_class)
+	{
+	  double d = (double) constExpr->v_const.val.fVal;
+	  if (!memcmp (&d, &OOP_TO_OBJ (oop)->data, sizeof (double)))
+	    return (true);
+	}
+      break;
+
     case CONST_FLOATE:
-      if (IS_OOP (oop) && OOP_CLASS (oop) == _gst_floate_class
-	  && (float) constExpr->v_const.val.fVal == FLOATE_OOP_VALUE (oop))
-	return (true);
+      if (IS_OOP (oop) && OOP_CLASS (oop) == _gst_floate_class)
+	{
+	  float f = (float) constExpr->v_const.val.fVal;
+	  if (!memcmp (&f, &OOP_TO_OBJ (oop)->data, sizeof (float)))
+	    return (true);
+	}
       break;
 
     case CONST_FLOATQ:
-      if (IS_OOP (oop) && OOP_CLASS (oop) == _gst_floatq_class
-	  && constExpr->v_const.val.fVal == FLOATQ_OOP_VALUE (oop))
-	return (true);
+      if (IS_OOP (oop) && OOP_CLASS (oop) == _gst_floatq_class)
+	{
+	  long double ld = (long double) constExpr->v_const.val.fVal;
+	  if (!memcmp (&ld, &OOP_TO_OBJ (oop)->data, sizeof (long double)))
+	    return (true);
+	}
       break;
 
     case CONST_STRING:
@@ -2114,7 +2132,7 @@ make_constant_oop (tree_node constExpr)
   OOP resultOOP, elementOOP;
   inc_ptr incPtr;
   byte_object bo;
-  mst_Object result;
+  gst_object result;
 
   if (constExpr == NULL)
     return (_gst_nil_oop);	/* special case empty array literals */
@@ -2154,6 +2172,9 @@ make_constant_oop (tree_node constExpr)
     case CONST_INT:
       return (FROM_INT (constExpr->v_const.val.iVal));
 
+    case CONST_CHAR:
+      return (char_new (constExpr->v_const.val.iVal));
+
     case CONST_FLOATD:
       return (floatd_new (constExpr->v_const.val.fVal));
 
@@ -2290,18 +2311,42 @@ _gst_make_attribute (tree_node attribute_keywords)
 {
   tree_node keyword;
   OOP selectorOOP, argsArrayOOP, messageOOP;
-  mst_Object argsArray;
+  gst_object argsArray;
   int i, numArgs;
+  inc_ptr incPtr;
+
+  incPtr = INC_SAVE_POINTER ();
   
+  if (_gst_had_error)
+    return _gst_nil_oop;
+
   selectorOOP = compute_keyword_selector (attribute_keywords);
   numArgs = list_length (attribute_keywords);
   argsArray = new_instance_with (_gst_array_class, numArgs, &argsArrayOOP);
+  INC_ADD_OOP (argsArrayOOP);
 
   for (i = 0, keyword = attribute_keywords; keyword != NULL;
        i++, keyword = keyword->v_list.next)
-    argsArray->data[i] = keyword->v_list.value->v_const.val.oopVal;
+    {
+      tree_node value = keyword->v_list.value;
+      if (value->nodeType != TREE_CONST_EXPR)
+	{
+          tree_node stmt = _gst_make_statement_list (&value->location, value);
+          OOP result = _gst_execute_statements (NULL, stmt, true);
+          value = _gst_make_oop_constant (&stmt->location, result);
+          if (!result)
+	    {
+	      _gst_had_error = true;
+	      return _gst_nil_oop;
+	    }
+	}
+
+      argsArray = OOP_TO_OBJ (argsArrayOOP);
+      argsArray->data[i] = make_constant_oop (value);
+    }
 
   messageOOP = _gst_message_new_args (selectorOOP, argsArrayOOP);
+  INC_RESTORE_POINTER (incPtr);
 
   MAKE_OOP_READONLY (argsArrayOOP, true);
   MAKE_OOP_READONLY (messageOOP, true);
@@ -2381,7 +2426,7 @@ process_attribute (OOP messageOOP)
   gst_message message = (gst_message) OOP_TO_OBJ (messageOOP);
   OOP selectorOOP = message->selector;
   OOP argumentsOOP = message->args;
-  mst_Object arguments = OOP_TO_OBJ (argumentsOOP);
+  gst_object arguments = OOP_TO_OBJ (argumentsOOP);
 
   if (selectorOOP == _gst_primitive_symbol)
     {
@@ -2484,7 +2529,7 @@ OOP
 get_literals_array (void)
 {
   OOP methodLiteralsOOP;
-  mst_Object methodLiterals;
+  gst_object methodLiterals;
 
   assert (literal_vec_curr > literal_vec);
 
@@ -2508,9 +2553,40 @@ install_method (OOP methodOOP)
   OOP oldMethod, selector, methodDictionaryOOP;
   gst_compiled_method method;
   gst_method_info descriptor;
+  int num_attrs, i;
 
   method = (gst_compiled_method) OOP_TO_OBJ (methodOOP);
   descriptor = (gst_method_info) OOP_TO_OBJ (method->descriptor);
+  num_attrs = NUM_INDEXABLE_FIELDS (method->descriptor);
+
+  for (i = 0; i < num_attrs; i++)
+    {
+      char *result;
+      OOP attributeOOP = descriptor->attributes[i];
+      gst_message attribute = (gst_message) OOP_TO_OBJ (attributeOOP);
+      OOP handlerBlockOOP = _gst_find_pragma_handler (_gst_this_class,
+						      attribute->selector);
+
+      if (!IS_NIL (handlerBlockOOP))
+	{
+	  _gst_msg_sendf (&result, "%s %o value: %o value: %o",
+			  handlerBlockOOP, methodOOP, attributeOOP);
+	  if (result != NULL)
+	    {
+	      _gst_errorf ("%s", result);
+	      EXIT_COMPILATION ();
+	    }
+	}
+
+      method = (gst_compiled_method) OOP_TO_OBJ (methodOOP);
+      descriptor = (gst_method_info) OOP_TO_OBJ (method->descriptor);
+      if (num_attrs != NUM_INDEXABLE_FIELDS (method->descriptor))
+	{
+	  _gst_errorf ("cannot modify method descriptor in pragma handler");
+	  EXIT_COMPILATION ();
+	}
+    }
+
   selector = descriptor->selector;
 
   /* methodDictionaryOOP is held onto by the class, which is already
@@ -2530,6 +2606,7 @@ install_method (OOP methodOOP)
 	}
     }
 
+  MAKE_OOP_READONLY (methodOOP, true);
   oldMethod = _gst_identity_dictionary_at_put (methodDictionaryOOP,
 					       selector, methodOOP);
 
@@ -2648,7 +2725,7 @@ method_new (method_header header,
   int numByteCodes;
   gst_compiled_method method;
   OOP methodOOP, methodDesc;
-  mst_Object lit;
+  gst_object lit;
   int i;
 
   if (bytecodes != NULL)
@@ -2699,8 +2776,6 @@ method_new (method_header header,
       _gst_free_bytecodes (bytecodes);
     }
 
-  MAKE_OOP_READONLY (methodOOP, true);
-
   return (methodOOP);
 }
 
@@ -2780,44 +2855,29 @@ method_info_new (OOP class,
 OOP
 file_segment_new (void)
 {
-  OOP fileName, stringContents, fileSegmentOOP;
+  OOP fileName, fileSegmentOOP;
   gst_file_segment fileSegment;
-  int startPos;
+  int64_t startPos = _gst_get_method_start_pos ();
   inc_ptr incPtr;
 
-  switch (_gst_get_cur_stream_type ())
-    {
-    case STREAM_UNKNOWN:
-    default:
-      return (_gst_nil_oop);
+  if (startPos == -1)
+    return _gst_get_cur_string ();
 
-    case STREAM_FILE:
-      incPtr = INC_SAVE_POINTER ();
-      fileName = _gst_get_cur_file_name ();
-      INC_ADD_OOP (fileName);
+  incPtr = INC_SAVE_POINTER ();
+  fileName = _gst_get_cur_file_name ();
+  INC_ADD_OOP (fileName);
 
-      fileSegment = (gst_file_segment) new_instance (_gst_file_segment_class,
-    						 &fileSegmentOOP);
+  fileSegment = (gst_file_segment) new_instance (_gst_file_segment_class,
+						 &fileSegmentOOP);
 
-      fileSegment->fileName = fileName;
-      startPos = _gst_get_method_start_pos ();
-      fileSegment->startPos = FROM_INT (startPos);
-      fileSegment->length =
-        FROM_INT (_gst_get_cur_file_pos () - startPos - 1);
+  fileSegment->fileName = fileName;
+  fileSegment->startPos = from_c_int_64 (startPos);
+  fileSegment->length =
+    from_c_int_64 (_gst_get_cur_file_pos () - startPos - 1);
 
-      INC_RESTORE_POINTER (incPtr);
-      return (fileSegmentOOP);
-
-    case STREAM_STRING:
-      stringContents = _gst_get_cur_string ();
-      return (stringContents);
-
-#ifdef HAVE_READLINE
-    case STREAM_READLINE:
-      stringContents = _gst_get_cur_readline ();
-      return (stringContents);
-#endif /* HAVE_READLINE */
-    }
+  assert (to_c_int_64 (fileSegment->length) >= 0);
+  INC_RESTORE_POINTER (incPtr);
+  return (fileSegmentOOP);
 }
 
 void _gst_restore_primitive_number (OOP methodOOP, int *map)
