@@ -28,7 +28,7 @@
  *
  * You should have received a copy of the GNU General Public License along with
  * GNU Smalltalk; see the file COPYING.	 If not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  ***********************************************************************/
 
@@ -97,23 +97,6 @@
 #define ASYNC_QUEUE_SIZE		100
 
 
-/* CompiledMethod cache (see descriptions in interp-bc.inl and
-   interp-jit.inl) */
-typedef struct method_cache_entry
-{
-  OOP selectorOOP;
-  OOP startingClassOOP;
-  OOP methodOOP;
-  OOP methodClassOOP;
-  method_header methodHeader;
-#ifdef ENABLE_JIT_TRANSLATION
-  OOP receiverClass;
-  PTR nativeCode;
-  PTR dummy;			/* 32 bytes are usually a sweet spot */
-#endif
-}
-method_cache_entry;
-
 typedef struct async_queue_entry
 {
   OOP sem;
@@ -229,10 +212,10 @@ static int last_primitive;
    enabled.  */
 #ifndef ENABLE_JIT_TRANSLATION
 static OOP at_cache_class;
-static int at_cache_prim;
+static intptr_t at_cache_spec;
 
 static OOP at_put_cache_class;
-static int at_put_cache_prim;
+static intptr_t at_put_cache_spec;
 
 static OOP size_cache_class;
 static int size_cache_prim;
@@ -253,7 +236,7 @@ const char *_gst_abort_execution = NULL;
 
 /* Set to true when some special action must be done at the next
    sequence point.  */
-mst_Boolean _gst_except_flag = false;
+volatile mst_Boolean _gst_except_flag = false;
 
 /* Set to non-nil if a process must preempt the current one.  */
 static OOP switch_to_process;
@@ -273,6 +256,26 @@ static interp_jmp_buf *reentrancy_jmp_buf = NULL;
    the stack is printed as well as the byte code */
 static mst_Boolean verbose_exec_tracing = false;
 
+/* This is the bridge to the primitive operations in the GNU Smalltalk
+   system.  This function invokes the proper primitive_func with the
+   correct id and the same NUMARGS and METHODOOP with which it was
+   invoked.  */
+static inline intptr_t execute_primitive_operation (int primitive,
+						    volatile int numArgs);
+
+/* Execute a #at: primitive, with arguments REC and IDX, knowing that
+   the receiver's class has an instance specification SPEC.  */
+static inline mst_Boolean cached_index_oop_primitive (OOP rec,
+						      OOP idx,
+						      intptr_t spec);
+
+/* Execute a #at:put: primitive, with arguments REC/IDX/VAL, knowing that
+   the receiver's class has an instance specification SPEC.  */
+static inline mst_Boolean cached_index_oop_put_primitive (OOP rec,
+							  OOP idx,
+							  OOP val,
+							  intptr_t spec);
+
 /* Locates in the ProcessorScheduler's process lists and returns the
    highest priority process different from the current process.  */
 static OOP highest_priority_process (void) ATTRIBUTE_PURE;
@@ -312,7 +315,7 @@ static void stop_executing (const char *msg);
 static void set_preemption_timer (void);
 
 /* Same as _gst_parse_stream, but creating a reentrancy_jmpbuf.  */
-static void parse_stream_with_protection ();
+static void parse_stream_with_protection (mst_Boolean method);
 
 /* Put the given process to sleep by rotating the list of processes for
    PROCESSOOP's priority (i.e. it was the head of the list and becomes
@@ -409,12 +412,6 @@ static mst_Boolean lookup_method (OOP sendSelector,
 				  int sendArgs,
 				  OOP method_class);
 
-/* This is a further simplified lookup_method which does not care
-   about preparing for #doesNotUnderstand:.  */
-static inline mst_Boolean find_method (OOP receiverClass,
-				       OOP sendSelector,
-				       method_cache_entry *methodData);
-
 /* This tenures context objects from the stack to the context pools
    (see below for a description).  */
 static void empty_context_stack (void);
@@ -487,7 +484,7 @@ static mst_Boolean unwind_method (void);
    or an unwind method.  In this case the non-unwind contexts between
    the unwind method and the returnContextOOP must be removed from the
    chain.  */
-static void unwind_to (OOP returnContextOOP);
+static mst_Boolean unwind_to (OOP returnContextOOP);
 
 /* Arrange things so that all the non-unwinding contexts up to
    returnContextOOP aren't executed.  For block contexts this can
@@ -496,7 +493,7 @@ static void unwind_to (OOP returnContextOOP);
    from them!  For this reason, method contexts are flagged as
    disabled and unwind_context takes care of skipping them when
    doing a local return.  */
-static void disable_non_unwind_contexts (OOP returnContextOOP);
+static mst_Boolean disable_non_unwind_contexts (OOP returnContextOOP);
 
 /* Called to handle signals that are not passed to the Smalltalk
    program, such as interrupts or segmentation violation.  In the
@@ -674,7 +671,7 @@ static gst_context_part *chunk = chunks - 1;
    and as long as no context switches happen since the time the
    process was created.  FREE_LIFO_CONTEXT points to just after the
    top of the stack.  */
-static struct OOP lifo_contexts[MAX_LIFO_DEPTH] CACHELINE_ALIGNED;
+static struct oop_s lifo_contexts[MAX_LIFO_DEPTH] CACHELINE_ALIGNED;
 static OOP free_lifo_context = lifo_contexts;
 
 /* Include `plug-in' modules for the appropriate interpreter.
@@ -890,7 +887,7 @@ dealloc_stack_context (gst_context_part context)
 {
 #ifndef OPTIMIZE
   if (free_lifo_context == lifo_contexts
-      || (OOP_TO_OBJ (free_lifo_context - 1) != (mst_Object) context))
+      || (OOP_TO_OBJ (free_lifo_context - 1) != (gst_object) context))
     {
       _gst_errorf ("Deallocating a non-LIFO context!!!");
       abort ();
@@ -936,7 +933,7 @@ lookup_method (OOP sendSelector,
   inc_ptr inc;
   OOP argsArrayOOP;
 
-  if (find_method (method_class, sendSelector, methodData))
+  if (_gst_find_method (method_class, sendSelector, methodData))
     return (true);
 
   inc = INC_SAVE_POINTER ();
@@ -948,9 +945,9 @@ lookup_method (OOP sendSelector,
 }
 
 mst_Boolean
-find_method (OOP receiverClass,
-	     OOP sendSelector,
-	     method_cache_entry *methodData)
+_gst_find_method (OOP receiverClass,
+	          OOP sendSelector,
+	          method_cache_entry *methodData)
 {
   OOP method_class = receiverClass;
   for (; !IS_NIL (method_class);
@@ -965,6 +962,12 @@ find_method (OOP receiverClass,
 	  methodData->methodOOP = methodOOP;
 	  methodData->methodClassOOP = method_class;
 	  methodData->methodHeader = GET_METHOD_HEADER (methodOOP);
+
+#ifdef ENABLE_JIT_TRANSLATION
+	  /* Force the translation to be looked up the next time
+	     this entry is used for a message send.  */
+	  methodData->receiverClass = NULL;
+#endif
 	  _gst_cache_misses++;
 	  return (true);
 	}
@@ -976,7 +979,7 @@ find_method (OOP receiverClass,
 OOP
 create_args_array (int numArgs)
 {
-  mst_Object argsArray;
+  gst_object argsArray;
   OOP argsArrayOOP;
   int i;
 
@@ -1006,7 +1009,7 @@ check_send_correctness (OOP receiver,
     {
       /* If we do not find the method, don't worry and fire
 	 #doesNotUnderstand:  */
-      if (!find_method (receiverClass, sendSelector, methodData))
+      if (!_gst_find_method (receiverClass, sendSelector, methodData))
 	return (true);
 
       methodData = &method_cache[hashIndex];
@@ -1117,12 +1120,11 @@ unwind_method (void)
       return (false);
     }
 
-  unwind_to (newContext->parentContext);
-  return (true);
+  return unwind_to (newContext->parentContext);
 }
 
 
-void
+mst_Boolean
 unwind_to (OOP returnContextOOP)
 {
   OOP oldContextOOP, newContextOOP;
@@ -1145,15 +1147,16 @@ unwind_to (OOP returnContextOOP)
       /* Check if we got to an unwinding context (#ensure:).  */
       if UNCOMMON (CONTEXT_FLAGS (newContext) & MCF_IS_UNWIND_CONTEXT)
         {
+	  mst_Boolean result;
 	  _gst_this_context_oop = oldContextOOP;
 
 	  /* _gst_this_context_oop is the context above the
 	     one we return to.   We only unwind up to the #ensure:
 	     context.  */
-	  disable_non_unwind_contexts (returnContextOOP);
+	  result = disable_non_unwind_contexts (returnContextOOP);
 
 	  unwind_context ();
-	  return;
+	  return result;
 	}
 
       /* This context cannot be deallocated in a LIFO way.  We must
@@ -1188,9 +1191,10 @@ unwind_to (OOP returnContextOOP)
   _gst_self = newContext->receiver;
 
   SET_THIS_METHOD (newContext->method, GET_CONTEXT_IP (newContext));
+  return (true);
 }
 
-void
+mst_Boolean
 disable_non_unwind_contexts (OOP returnContextOOP)
 {
   OOP oldContextOOP, newContextOOP, *chain;
@@ -1215,6 +1219,12 @@ disable_non_unwind_contexts (OOP returnContextOOP)
 	   collect more context objects.  */
         oldContext->parentContext = _gst_nil_oop;
 
+      if (IS_NIL (newContextOOP))
+	{
+	  *chain = newContextOOP;
+	  return (false);
+	}
+
       if (newContextOOP == returnContextOOP)
 	{
 	  *chain = newContextOOP;
@@ -1252,6 +1262,7 @@ disable_non_unwind_contexts (OOP returnContextOOP)
     }
 
   *chain = newContext->parentContext;
+  return (true);
 }
 
 
@@ -1296,7 +1307,7 @@ change_process_context (OOP newProcess)
   switch_to_process = _gst_nil_oop;
 
   /* save old context information */
-  if (ip)
+  if (!IS_NIL (_gst_this_context_oop))
     empty_context_stack ();
 
   /* printf("Switching to process %#O at priority %#O\n",
@@ -1348,6 +1359,10 @@ change_process_context (OOP newProcess)
   SET_THIS_METHOD (thisContext->method, GET_CONTEXT_IP (thisContext));
   sp = thisContext->contextStack + TO_INT (thisContext->spOffset);
 
+#if ENABLE_JIT_TRANSLATION
+  ip = TO_INT (thisContext->ipOffset);
+#endif
+
   _gst_temporaries = thisContext->contextStack;
   _gst_self = thisContext->receiver;
 }
@@ -1934,7 +1949,7 @@ _gst_init_process_system (void)
   processor = (gst_processor_scheduler) OOP_TO_OBJ (_gst_processor_oop);
   if (IS_NIL (processor->processLists))
     {
-      mst_Object processLists;
+      gst_object processLists;
 
       processLists = instantiate_with (_gst_array_class, NUM_PRIORITIES,
 				       &processor->processLists);
@@ -2051,10 +2066,9 @@ _gst_prepare_execution_environment (void)
   dummyContext->spOffset = FROM_INT (-1);
 
 #ifdef ENABLE_JIT_TRANSLATION
-  dummyContext->native_ip = FROM_INT ((char *) _gst_return_from_native_code);
+  dummyContext->native_ip = GET_NATIVE_IP ((char *) _gst_return_from_native_code);
 #else
-  dummyContext->native_ip = DUMMY_NATIVE_IP;	/* See
-						   empty_context_stack */
+  dummyContext->native_ip = DUMMY_NATIVE_IP;	/* See empty_context_stack */
 #endif
 
   dummyContextOOP = alloc_oop (dummyContext,
@@ -2272,7 +2286,7 @@ _gst_fixup_object_pointers (void)
 {
   gst_method_context thisContext;
 
-  if (ip)
+  if (!IS_NIL (_gst_this_context_oop))
     {
       /* Create real OOPs for the contexts here.  If we do it while copying,
          the newly created OOPs are in to-space and are never scanned! */
@@ -2306,7 +2320,7 @@ _gst_restore_object_pointers (void)
      and we also pick up the context to adjust sp and the temps
      accordingly.  */
 
-  if (ip)
+  if (!IS_NIL (_gst_this_context_oop))
     {
       thisContext =
 	(gst_context_part) OOP_TO_OBJ (_gst_this_context_oop);
@@ -2356,6 +2370,7 @@ _gst_init_signals (void)
 #endif
       _gst_set_signal_handler (SIGABRT, interrupt_handler);
     }
+  _gst_set_signal_handler (SIGTERM, interrupt_handler);
   _gst_set_signal_handler (SIGINT, interrupt_handler);
   _gst_set_signal_handler (SIGFPE, interrupt_handler);
   _gst_set_signal_handler (SIGUSR1, interrupt_handler);
@@ -2381,6 +2396,10 @@ interrupt_handler (int sig)
 
   switch (sig)
     {
+    case SIGTERM:
+      is_serious_error = false;
+      break;
+
     case SIGUSR1:
       is_serious_error = false;
       _gst_set_signal_handler (sig, interrupt_handler);
@@ -2392,7 +2411,7 @@ interrupt_handler (int sig)
 
     case SIGINT:
       is_serious_error = false;
-      if (!_gst_non_interactive)
+      if (!_gst_non_interactive && in_interpreter)
 	{
 	  _gst_set_signal_handler (sig, interrupt_handler);
 	  stop_executing ("userInterrupt");
@@ -2423,7 +2442,8 @@ interrupt_handler (int sig)
 #ifdef HAVE_EXECINFO_H
       /* Don't print a backtrace, for example, if exiting during a
 	 compilation.  */
-      if (ip || _gst_gc_running || is_serious_error || sig == SIGUSR1)
+      if ((_gst_verbosity > 2 && (ip || _gst_gc_running))
+	  || is_serious_error || sig == SIGUSR1)
 	{
           PTR array[11];
           size_t size = backtrace (array, 11);
@@ -2432,11 +2452,19 @@ interrupt_handler (int sig)
 #endif
     }
 
-  if (sig == SIGUSR1)
-    return;
+  switch (sig)
+    {
+    case SIGUSR1:
+      return;
 
-  _gst_set_signal_handler (sig, SIG_DFL);
-  raise (sig);
+    case SIGTERM:
+    case SIGINT:
+      exit (0);
+
+    default:
+      _gst_set_signal_handler (sig, SIG_DFL);
+      raise (sig);
+    }
 }
 
 void
@@ -2457,9 +2485,8 @@ _gst_show_backtrace (void)
 	  == (MCF_IS_METHOD_CONTEXT | MCF_IS_DISABLED_CONTEXT))
 	continue;
 
-#if 0
-      printf ("(OOP %p)", context->method);
-#endif
+      /* printf ("(OOP %p)", context->method); */
+      printf ("(ip %d)", TO_INT (context->ipOffset));
       if (CONTEXT_FLAGS (context) & MCF_IS_METHOD_CONTEXT)
 	{
 	  OOP receiver, receiverClass;
@@ -2529,8 +2556,39 @@ _gst_show_stack_contents (void)
 }
 
 
-intptr_t _gst_execute_primitive_operation (int primitive,
-                                           volatile int numArgs)
+static inline mst_Boolean
+cached_index_oop_primitive (OOP rec, OOP idx, intptr_t spec)
+{
+  OOP result;
+  if (!IS_INT (idx))
+    return (true);
+
+  result = index_oop_spec (rec, OOP_TO_OBJ (rec), TO_INT (idx), spec);
+  if UNCOMMON (!result)
+    return (true);
+
+  POP_N_OOPS (1);
+  SET_STACKTOP (result);
+  return (false);
+}
+
+static inline mst_Boolean
+cached_index_oop_put_primitive (OOP rec, OOP idx, OOP val, intptr_t spec)
+{
+  if (!IS_INT (idx))
+    return (true);
+
+  if UNCOMMON (!index_oop_put_spec (rec, OOP_TO_OBJ (rec), TO_INT (idx),
+				    val, spec))
+    return (true);
+
+  POP_N_OOPS (2);
+  SET_STACKTOP (val);
+  return (false);
+}
+
+static inline intptr_t
+execute_primitive_operation (int primitive, volatile int numArgs)
 {
   prim_table_entry *pte = &_gst_primitive_table[primitive];
 
@@ -2546,10 +2604,10 @@ _gst_get_primitive_attributes (int primitive)
 }
 
 void
-parse_stream_with_protection ()
+parse_stream_with_protection (mst_Boolean method)
 {
   interp_jmp_buf localJmpBuf;
 
   PROTECT_CURRENT_PROCESS_WITH (&localJmpBuf)
-    _gst_parse_stream ();
+    _gst_parse_stream (method);
 }
