@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2004,2005,2006
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -25,7 +25,7 @@
  * 
  * You should have received a copy of the GNU General Public License along with
  * GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
+ * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  *
  ***********************************************************************/
 
@@ -54,14 +54,6 @@
 #define BIN_OP_CHAR		8
 #define SYMBOL_CHAR		16
 
-#if defined(WIN32) || defined(__OS2__) || defined(MSDOS)
-#define HAVE_DOS_STYLE_PATH_NAMES
-#endif
-
-/* This is necessary so that the grammar knows when it must switch to
-   compile mode */
-mst_Boolean _gst_compile_code = false;
-
 /* The obstack containing parse tree nodes.  */
 struct obstack *_gst_compilation_obstack = NULL;
 
@@ -76,11 +68,6 @@ char *_gst_first_error_str = NULL;
 char *_gst_first_error_file = NULL;
 int _gst_first_error_line = 0;
 
-/* The starting-position state.  If this is <0, as soon as another
-   token is lexed, the flag is reinitialized to the starting position
-   of the next token.  */
-static int method_start_pos = -1;
-
 /* Answer true if IC is a valid base-10 digit.  */
 static mst_Boolean is_digit (int ic);
 
@@ -94,7 +81,7 @@ static mst_Boolean is_base_digit (int c,
    is not NULL, the digits are stored in the buffer maintained
    by str.c, and LARGEINTEGER is set to true if numPtr does
    not have sufficient precision.  */
-static int parse_fraction (int c,
+static int scan_fraction (int c,
 			   int base,
 			   long double *numPtr,
 			   mst_Boolean *largeInteger);
@@ -106,7 +93,7 @@ static int parse_fraction (int c,
    digits are stored in the buffer maintained by str.c,
    and LARGEINTEGER is set to true if the return value does
    not have sufficient precision.  */
-static long double parse_digits (int c,
+static long double scan_digits (int c,
 			    mst_Boolean negative,
 			    int base,
 			    mst_Boolean * largeInteger);
@@ -115,7 +102,7 @@ static long double parse_digits (int c,
    digits in the buffer maintained by str.c, adjusting
    the sign if NEGATIVE is true.  Return an embryo of the
    LargeInteger object as a byte_object structure.  */
-static byte_object parse_large_integer (mst_Boolean negative,
+static byte_object scan_large_integer (mst_Boolean negative,
 					int base);
 
 /* Raise an error.  */
@@ -132,7 +119,7 @@ static int char_literal (int c,
 			 YYSTYPE * lvalp);
 
 /* Parse a binary operator.  C is the first symbol in the selector */
-static int parse_bin_op (int c,
+static int scan_bin_op (int c,
 			 YYSTYPE * lvalp);
 
 /* Parse a string literal.  C is '\'' */
@@ -140,19 +127,19 @@ static int string_literal (int c,
 			   YYSTYPE * lvalp);
 
 /* Parse a number.  C is the first digit.  */
-static int parse_number (int c,
+static int scan_number (int c,
 			 YYSTYPE * lvalp);
 
 /* Parse an identifier.  C is the first letter.  */
-static int parse_ident (int c,
+static int scan_ident (int c,
 			YYSTYPE * lvalp);
 
 /* Try to parse an assignment operator or namespace separator.  C is ':'.  */
-static int parse_colon (int c,
+static int scan_colon (int c,
 			YYSTYPE * lvalp);
 
 /* Try to parse a symbol constant, or return '#'.  C is '#'.  */
-static int parse_symbol (int c,
+static int scan_symbol (int c,
 			 YYSTYPE * lvalp);
 
 /* Convert the digit C (if it is a valid base-BASE digit) to its
@@ -217,98 +204,98 @@ static const lex_tab_elt char_table[128] = {
 /*     */ {0, 0, WHITE_SPACE},
 /*   ! */ {0, '!', 0},
 /*   " */ {comment, 0, 0},
-/*   # */ {parse_symbol, 0, 0},
+/*   # */ {scan_symbol, 0, 0},
 /*   $ */ {char_literal, 0, ID_CHAR | SYMBOL_CHAR},
-/*   % */ {parse_bin_op, 0, BIN_OP_CHAR},
-/*   & */ {parse_bin_op, 0, BIN_OP_CHAR},
+/*   % */ {scan_bin_op, 0, BIN_OP_CHAR},
+/*   & */ {scan_bin_op, 0, BIN_OP_CHAR},
 /*   ' */ {string_literal, 0, 0},
 /*   ( */ {0, '(', 0},
 /*   ) */ {0, ')', 0},
-/*   * */ {parse_bin_op, 0, BIN_OP_CHAR},
-/*   + */ {parse_bin_op, 0, BIN_OP_CHAR},
-/*   , */ {parse_bin_op, 0, BIN_OP_CHAR},
-/*   - */ {parse_bin_op, 0, BIN_OP_CHAR},
+/*   * */ {scan_bin_op, 0, BIN_OP_CHAR},
+/*   + */ {scan_bin_op, 0, BIN_OP_CHAR},
+/*   , */ {scan_bin_op, 0, BIN_OP_CHAR},
+/*   - */ {scan_bin_op, 0, BIN_OP_CHAR},
 /*   . */ {0, '.', 0},
-/*   / */ {parse_bin_op, 0, BIN_OP_CHAR},
-/*   0 */ {parse_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
-/*   1 */ {parse_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
-/*   2 */ {parse_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
-/*   3 */ {parse_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
-/*   4 */ {parse_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
-/*   5 */ {parse_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
-/*   6 */ {parse_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
-/*   7 */ {parse_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
-/*   8 */ {parse_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
-/*   9 */ {parse_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
-/*   : */ {parse_colon, 0, SYMBOL_CHAR},
+/*   / */ {scan_bin_op, 0, BIN_OP_CHAR},
+/*   0 */ {scan_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
+/*   1 */ {scan_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
+/*   2 */ {scan_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
+/*   3 */ {scan_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
+/*   4 */ {scan_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
+/*   5 */ {scan_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
+/*   6 */ {scan_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
+/*   7 */ {scan_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
+/*   8 */ {scan_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
+/*   9 */ {scan_number, 0, DIGIT | ID_CHAR | SYMBOL_CHAR},
+/*   : */ {scan_colon, 0, SYMBOL_CHAR},
 /*   ; */ {0, ';', 0},
-/*   < */ {parse_bin_op, 0, BIN_OP_CHAR},
-/*   = */ {parse_bin_op, 0, BIN_OP_CHAR},
-/*   > */ {parse_bin_op, 0, BIN_OP_CHAR},
-/*   ? */ {parse_bin_op, 0, BIN_OP_CHAR},
-/*   @ */ {parse_bin_op, 0, BIN_OP_CHAR},
-/*   A */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   B */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   C */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   D */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   E */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   F */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   G */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   H */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   I */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   J */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   K */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   L */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   M */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   N */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   O */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   P */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   Q */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   R */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   S */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   T */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   U */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   V */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   W */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   X */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   Y */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   Z */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   < */ {scan_bin_op, 0, BIN_OP_CHAR},
+/*   = */ {scan_bin_op, 0, BIN_OP_CHAR},
+/*   > */ {scan_bin_op, 0, BIN_OP_CHAR},
+/*   ? */ {scan_bin_op, 0, BIN_OP_CHAR},
+/*   @ */ {scan_bin_op, 0, BIN_OP_CHAR},
+/*   A */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   B */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   C */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   D */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   E */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   F */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   G */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   H */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   I */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   J */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   K */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   L */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   M */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   N */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   O */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   P */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   Q */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   R */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   S */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   T */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   U */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   V */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   W */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   X */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   Y */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   Z */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
 /*   [ */ {0, '[', 0},
-/*   \ */ {parse_bin_op, 0, BIN_OP_CHAR},
+/*   \ */ {scan_bin_op, 0, BIN_OP_CHAR},
 /*   ] */ {0, ']', 0},
 /*   ^ */ {0, '^', 0},
 /*   _ */ {0, ASSIGNMENT, ID_CHAR | SYMBOL_CHAR},
 /*   ` */ {invalid, 0, 0},
-/*   a */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   b */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   c */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   d */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   e */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   f */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   g */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   h */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   i */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   j */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   k */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   l */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   m */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   n */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   o */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   p */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   q */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   r */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   s */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   t */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   u */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   v */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   w */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   x */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   y */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
-/*   z */ {parse_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   a */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   b */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   c */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   d */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   e */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   f */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   g */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   h */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   i */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   j */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   k */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   l */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   m */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   n */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   o */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   p */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   q */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   r */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   s */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   t */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   u */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   v */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   w */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   x */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   y */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
+/*   z */ {scan_ident, 0, ID_CHAR | SYMBOL_CHAR},
 /*   { */ {0, '{', 0},
-/*   | */ {parse_bin_op, 0, BIN_OP_CHAR},
+/*   | */ {scan_bin_op, 0, BIN_OP_CHAR},
 /*   } */ {0, '}', 0},
-/*   ~ */ {parse_bin_op, 0, BIN_OP_CHAR},
+/*   ~ */ {scan_bin_op, 0, BIN_OP_CHAR},
 /*  ^? */ {invalid, 0, 0}
 };
 
@@ -332,66 +319,29 @@ _gst_yylex (PTR lvalp, YYLTYPE *llocp)
 int
 _gst_yylex (PTR lvalp, YYLTYPE *llocp)
 {
-  int ic, result, tokenStartPos = 0;
+  int ic, result;
   const lex_tab_elt *ct;
 
-  if (_gst_compile_code)
-    {
-      /* The internal token is a trick to make the grammar be
-         "conditional". Normally, the allowable syntax for internal
-         compilation is that of a single method (without the
-         terminating '!').  However, would make for an ambiguous
-         grammar since a method declaration could look like an
-         immediate expression.  So, when the compiler is called
-         internally, we force the first token returned by the lexer to
-         be INTERNAL_TOKEN and make the top most production in the
-         grammar use INTERNAL_TOKEN as the first token of an internal
-         method. The same happens when you have a call to methodsFor:
-         - INTERNAL_TOKEN is pushed so that the grammar switches to
-         the method_list production.  */
-      _gst_compile_code = false;
-      return (INTERNAL_TOKEN);
-    }
-
   while ((ic = _gst_next_char ()) != EOF)
     {
-
       ct = CHAR_TAB (ic);
       if ((ct->char_class & WHITE_SPACE) == 0)
 	{
 	  _gst_get_location (&llocp->first_column, &llocp->first_line);
-	  if (method_start_pos < 0)
-	    tokenStartPos = _gst_get_cur_file_pos ();
 
+	  assert (ct->lexFunc || ct->retToken);
 	  if (ct->lexFunc)
 	    {
 	      result = (*ct->lexFunc) (ic, (YYSTYPE *) lvalp);
-	      _gst_get_location (&llocp->last_column, &llocp->last_line);
 	      if (result)
-		{
-		  if (method_start_pos < 0)
-		    /* only do this here to ignore comments */
-		    method_start_pos = tokenStartPos - 1;
-
-		  return (result);
-		}
+		return (result);
 	    }
 	  else if (ct->retToken)
-	    {
-	      llocp->last_line = llocp->first_line;
-	      llocp->last_column = llocp->first_column;
-	      return (ct->retToken);
-	    }
-
-	  else
-	    {
-	      _gst_errorf
-		("Unknown character %d in input stream, ignoring", ic);
-	      _gst_had_error = true;
-	    }
+	    return (ct->retToken);
 	}
     }
-  return (0);			/* keep fussy compilers happy */
+
+  return (EOF);
 }
 
 
@@ -460,13 +410,13 @@ char_literal (int c,
     }
   else
     {
-      lvalp->cval = ic;
+      lvalp->ival = ic;
       return (CHAR_LITERAL);
     }
 }
 
 int
-parse_colon (int c,
+scan_colon (int c,
 	     YYSTYPE * lvalp)
 {
   int ic;
@@ -484,7 +434,7 @@ parse_colon (int c,
 
 
 int
-parse_symbol (int c,
+scan_symbol (int c,
 	      YYSTYPE *lvalp)
 {
   int ic;
@@ -493,10 +443,24 @@ parse_symbol (int c,
   if (ic == EOF)
     return '#';
 
+  /* Look for a shebang (#! /).  */
+  if (ic == '!')
+    {
+      int line, column;
+      _gst_get_location (&column, &line);
+      if (line == 1 && column == 2)
+        {
+          while (((ic = _gst_next_char ()) != EOF)
+                 && ic != '\r' && ic != '\n')
+            continue;
+          return (SHEBANG);
+        }
+    }
+
   /* We can read a binary operator and return a SYMBOL_LITERAL,... */
   if (CHAR_TAB (ic)->char_class & BIN_OP_CHAR)
     {
-      parse_bin_op (ic, lvalp);
+      scan_bin_op (ic, lvalp);
       return SYMBOL_LITERAL;
     }
 
@@ -528,7 +492,7 @@ parse_symbol (int c,
 
 
 int
-parse_bin_op (int c,
+scan_bin_op (int c,
 	      YYSTYPE *lvalp)
 {
   char buf[3];
@@ -561,7 +525,7 @@ parse_bin_op (int c,
       /* We come here also for a negative number, which we handle
          specially.  */
       if (c == '-' && is_digit (ic))
-	return (parse_number ('-', lvalp));
+	return (scan_number ('-', lvalp));
 
       buf[1] = 0;
     }
@@ -610,7 +574,7 @@ string_literal (int c,
 }
 
 int
-parse_ident (int c,
+scan_ident (int c,
 	     YYSTYPE * lvalp)
 {
   int ic, identType;
@@ -679,7 +643,7 @@ ipowl (long double base,
 }
 
 int
-parse_number (int c,
+scan_number (int c,
 	      YYSTYPE * lvalp)
 {
   OOP numOOP;
@@ -695,7 +659,7 @@ parse_number (int c,
 
   if (ic != '-')
     {
-      num = parse_digits (ic, isNegative, 10, &largeInteger);
+      num = scan_digits (ic, isNegative, 10, &largeInteger);
       ic = _gst_next_char ();
       if (ic == 'r')
 	{
@@ -726,7 +690,7 @@ parse_number (int c,
 	  ic = _gst_next_char ();
 	}
 
-      num = parse_digits (ic, isNegative, base, &largeInteger);
+      num = scan_digits (ic, isNegative, base, &largeInteger);
       ic = _gst_next_char ();
     }
 
@@ -745,7 +709,7 @@ parse_number (int c,
       else
 	{
 	  float_type = FLOATD_LITERAL;
-	  exponent = parse_fraction (ic, base, &num, &largeInteger);
+	  exponent = scan_fraction (ic, base, &num, &largeInteger);
 	  ic = _gst_next_char ();
 	}
     }
@@ -763,7 +727,7 @@ parse_number (int c,
 	else if (CHAR_TAB (ic)->char_class & DIGIT)
 	  {
 	    /* 123s4 format -- parse the exponent */
-	    floatExponent = parse_digits (ic, false, 10, NULL);
+	    floatExponent = scan_digits (ic, false, 10, NULL);
 	  }
 	else if (CHAR_TAB (ic)->char_class & ID_CHAR)
 	  {
@@ -782,8 +746,8 @@ parse_number (int c,
           {
 	    /* Make a LargeInteger constant and create an object out of
 	       it.  */
-	    byte_object bo = parse_large_integer (isNegative, base);
-	    mst_Object result = instantiate_with (bo->class, bo->size, &numOOP);
+	    byte_object bo = scan_large_integer (isNegative, base);
+	    gst_object result = instantiate_with (bo->class, bo->size, &numOOP);
             memcpy (result->data, bo->body, bo->size);
 	  }
         else
@@ -823,12 +787,12 @@ parse_number (int c,
         ;
       else if (ic == '-') {
 	  floatExponent =
-	    parse_digits (_gst_next_char (), true, 10, NULL);
+	    scan_digits (_gst_next_char (), true, 10, NULL);
 	  exponent -= (int) floatExponent;
 	}
       else if (CHAR_TAB (ic)->char_class & DIGIT)
 	{
-	  floatExponent = parse_digits (ic, false, 10, NULL);
+	  floatExponent = scan_digits (ic, false, 10, NULL);
 	  exponent += (int) floatExponent;
 	}
       else if (CHAR_TAB (ic)->char_class & ID_CHAR)
@@ -863,15 +827,9 @@ parse_number (int c,
     }
   else if (largeInteger || num < MIN_ST_INT || num > MAX_ST_INT)
     {
-      lvalp->boval = parse_large_integer (isNegative, base);
+      lvalp->boval = scan_large_integer (isNegative, base);
       return (LARGE_INTEGER_LITERAL);
     }
-  else if (num >= 0 && num <= 255)
-    {
-      lvalp->ival = (intptr_t) num;
-      return (BYTE_LITERAL);
-    }
-  else
     {
       lvalp->ival = (intptr_t) num;
       return (INTEGER_LITERAL);
@@ -879,7 +837,7 @@ parse_number (int c,
 }
 
 long double
-parse_digits (int c,
+scan_digits (int c,
 	      mst_Boolean negative,
 	      int base,
 	      mst_Boolean * largeInteger)
@@ -923,7 +881,7 @@ parse_digits (int c,
 }
 
 int
-parse_fraction (int c,
+scan_fraction (int c,
 		int base,
 		long double *numPtr,
 		mst_Boolean *largeInteger)
@@ -1012,7 +970,7 @@ is_digit (int ic)
 }
 
 byte_object
-parse_large_integer (mst_Boolean negative,
+scan_large_integer (mst_Boolean negative,
 		     int base)
 {
   int i;
@@ -1035,7 +993,7 @@ parse_large_integer (mst_Boolean negative,
   memzero (result, size);
 
   /* On each pass, multiply the previous partial result by the base,
-     and sum each of the digits as they were retrieved by parse_digits. 
+     and sum each of the digits as they were retrieved by scan_digits. 
    */
   for (digitsLeft = size; digitsLeft--;)
     {
@@ -1085,7 +1043,7 @@ parse_large_integer (mst_Boolean negative,
 
 
 void
-_gst_parse_stream (void)
+_gst_parse_stream (mst_Boolean method)
 {
   struct obstack thisObstack, *oldObstack;
 
@@ -1101,7 +1059,13 @@ _gst_parse_stream (void)
     while (_gst_yylex (&yylval));
 #else /* !NO_PARSE */
     _gst_had_error = false;
-    _gst_yyparse ();
+    if (method)
+      {
+	_gst_parse_method ();
+	_gst_reset_compilation_category ();
+      }
+    else
+      _gst_parse_chunks ();
 #endif /* !NO_PARSE */
   }
 
@@ -1110,18 +1074,6 @@ _gst_parse_stream (void)
 }
 
 
-int
-_gst_get_method_start_pos (void)
-{
-  return (method_start_pos);
-}
-
-void
-_gst_clear_method_start_pos (void)
-{
-  method_start_pos = -1;
-}
-
 
 #ifdef LEXDEBUG
 void
@@ -1155,9 +1107,6 @@ print_token (token,
     case ASSIGNMENT:
       printf (":=\n");
       break;
-    case INTERNAL_TOKEN:
-      printf ("INTERNAL_TOKEN\n");
-      break;
     case IDENTIFIER:
       printf ("IDENTIFIER: `%s'\n", yylval->sval);
       break;
@@ -1182,7 +1131,10 @@ print_token (token,
       printf ("FLOATQ_LITERAL: %Lg\n", yylval->fval);
       break;
     case CHAR_LITERAL:
-      printf ("CHAR_LITERAL: %d ($%c)\n", yylval->cval, yylval->cval);
+      printf ("CHAR_LITERAL: %d", yylval->ival, 
+      if (yylval->ival >= 32 && yylval->ival <= 126)
+	printf (" ($%c)", (char) yylval->ival);
+      printf ("\n");
       break;
     case STRING_LITERAL:
       printf ("STRING_LITERAL: '%s'\n", yylval->sval);
@@ -1227,7 +1179,10 @@ _gst_yyprint (FILE * file,
       fprintf (file, ": %Lg", yylval->fval);
       break;
     case CHAR_LITERAL:
-      fprintf (file, ": %d ($%c)\n", yylval->cval, yylval->cval);
+      fprintf (file, ": %d", yylval->ival);
+      if (yylval->ival >= 32 && yylval->ival <= 126)
+	fprintf (file, " ($%c)", (char) yylval->ival);
+      fprintf (file, "\n");
       break;
     default:
       break;
